@@ -57,7 +57,9 @@ use std::process::{Command, Stdio};
 
 use which::which;
 
-use crate::{error, Result};
+use crate::errors::clipboard::ClipboardError;
+use crate::errors::io::IoError;
+use crate::Result;
 
 /// The clipboard structure, holds the content string
 #[derive(Clone, Debug)]
@@ -67,23 +69,23 @@ pub struct Clipboard {
 
 /// The clipboard with the display server figured out
 #[derive(Clone, Debug)]
-pub struct Selected {
+pub struct ClipboardSelected {
     display: DisplayKind,
     content: String,
 }
 
 /// The clipboard that attempts the external binary approach
 #[derive(Clone, Debug)]
-pub struct Binary {
+pub struct BinClipboard {
     bin: OsString,
-    selected: Selected,
+    selected: ClipboardSelected,
     program: ClipboardBinProgram,
 }
 
 /// The clipboard that attempts OSC52 escape sequence approach
 #[derive(Clone, Debug)]
-pub struct EscapeSequence {
-    selected: Selected,
+pub struct EscapeSeqClipboard {
+    selected: ClipboardSelected,
 }
 
 /// The display server type
@@ -194,10 +196,10 @@ impl Clipboard {
     /// # Errors
     ///
     /// Fails with [`ClipboardError`] error
-    pub fn try_into_selected(self) -> Result<Selected> {
+    pub fn try_into_selected(self) -> Result<ClipboardSelected> {
         match select_display() {
-            DisplayKind::Unknown => Err(error::Clipboard::UnsupportedPlatform.into()),
-            valid => Ok(Selected {
+            DisplayKind::Unknown => Err(ClipboardError::UnknownPlatform.into()),
+            valid => Ok(ClipboardSelected {
                 display: valid,
                 content: self.content,
             }),
@@ -206,7 +208,7 @@ impl Clipboard {
 }
 
 /// The trait that a ready-to-use clipboard implements
-pub trait Provider {
+pub trait ClipboardProvider {
     /// Attempt to set the contents into the system clipboard
     ///
     /// # Errors
@@ -215,7 +217,7 @@ pub trait Provider {
     fn set_contents(&self) -> Result<()>;
 }
 
-impl Provider for Binary {
+impl ClipboardProvider for BinClipboard {
     fn set_contents(&self) -> Result<()> {
         let mut command = Command::new(&self.bin);
         match self.program {
@@ -231,32 +233,42 @@ impl Provider for Binary {
             .stdin(Stdio::piped())
             .stdout(Stdio::null())
             .stderr(Stdio::null())
-            .spawn()?;
+            .spawn()
+            .map_err(|err| ClipboardError::BinExecution(IoError::ProcessSpawn(err.to_string())))?;
 
         process
             .stdin
             .as_mut()
             .expect("to access stdin")
-            .write_all(self.selected.content.as_bytes())?;
+            .write_all(self.selected.content.as_bytes())
+            .map_err(|err| ClipboardError::BinExecution(IoError::StdinWrite(err.to_string())))?;
 
-        let _status = process.wait()?;
-
-        Ok(())
+        let status = process
+            .wait()
+            .map_err(|err| ClipboardError::BinExecution(IoError::ProcessWait(err.to_string())))?;
+        if status.success() {
+            Ok(())
+        } else {
+            Err(ClipboardError::BinExecution(IoError::Other(
+                "process returned non zero status".to_owned(),
+            ))
+            .into())
+        }
     }
 }
 
-impl Provider for EscapeSequence {
+impl ClipboardProvider for EscapeSeqClipboard {
     fn set_contents(&self) -> Result<()> {
         print!("\x1B]52;c;{}\x07", base64::encode(&self.selected.content));
         Ok(())
     }
 }
 
-impl Selected {
+impl ClipboardSelected {
     /// Transforms this clipboard into a ready-to-use kind
     /// First checks for binaries and fallbacks to the ANSI escape sequence approach.
     #[must_use]
-    pub fn into_provider(self) -> Box<dyn Provider> {
+    pub fn into_provider(self) -> Box<dyn ClipboardProvider> {
         match self.try_into_bin() {
             Ok(bin_clipboard) => {
                 return Box::new(bin_clipboard);
@@ -265,7 +277,7 @@ impl Selected {
                 println!("{:?}", err);
             }
         }
-        Box::new(EscapeSequence { selected: self })
+        Box::new(EscapeSeqClipboard { selected: self })
     }
 }
 
@@ -285,7 +297,7 @@ enum ClipboardBinProgram {
     target_family = "unix",
     not(all(target_os = "macos", target_os = "ios", target_os = "android"))
 ))]
-impl Selected {
+impl ClipboardSelected {
     /// Checks for supported clipboard binaries and attempts to convert the selected clipboard into
     /// the binary implementation.
     ///
@@ -293,7 +305,7 @@ impl Selected {
     ///
     /// Will fail with [`ClipboardError`] when any matched display server misses it's supported
     /// clipboard binaries.
-    fn try_into_bin(&self) -> Result<Binary> {
+    fn try_into_bin(&self) -> Result<BinClipboard> {
         let (bin, program) = match self.display {
             DisplayKind::X11 => {
                 let mut binaries = [
@@ -305,21 +317,24 @@ impl Selected {
 
                 let (bin, program) = binaries
                     .find(|(bin, _)| bin.is_ok())
-                    .ok_or(error::Clipboard::MissingBinary)?;
+                    .ok_or(ClipboardError::MissingX11ClipboardBin)?;
                 // Safe to unwrap since we previously checked `bin.is_ok()`
                 (bin.unwrap(), program)
             }
             DisplayKind::Wayland => {
-                let bin = which("wl-copy")?;
+                let bin =
+                    which("wl-copy").map_err(|_| ClipboardError::MissingWaylandClipboardBin)?;
                 let program = ClipboardBinProgram::WlCopy;
                 (bin, program)
             }
             DisplayKind::SshTty => {
                 //`xauth` missing most likely mean display passthrough isn't working
-                let _xauth = which("xauth")?;
+                let _xauth = which("xauth").map_err(|_| ClipboardError::MissingTtyClipboardBin)?;
 
                 // DISPALY variable different than `localhost:...` is a bad sign as well
-                env::var("DISPLAY").map_err(|_| error::Clipboard::DisplayNotSet)?;
+                let _display = env::var("DISPLAY")
+                    .map(|var| var.contains("localhost"))
+                    .map_err(|_| ClipboardError::MissingDisplayEnvSsh)?;
 
                 let mut binaries = [
                     (which("xclip"), ClipboardBinProgram::Xclip),
@@ -330,7 +345,7 @@ impl Selected {
 
                 let (bin, program) = binaries
                     .find(|(bin, _)| bin.is_ok())
-                    .ok_or(error::Clipboard::MissingBinary)?;
+                    .ok_or(ClipboardError::MissingX11ClipboardBin)?;
                 // Safe to unwrap since we previously checked `bin.is_ok()`
                 (bin.unwrap(), program)
             }
@@ -341,7 +356,7 @@ impl Selected {
             }
             DisplayKind::Unknown => panic!("clipboard feature not supported"),
         };
-        Ok(Binary {
+        Ok(BinClipboard {
             bin: bin.as_os_str().to_owned(),
             selected: self.clone(),
             program,
@@ -350,7 +365,7 @@ impl Selected {
 }
 
 #[cfg(all(target_os = "macos", target_os = "ios"))]
-impl Selected {
+impl ClipboardSelected {
     /// Checks for supported clipboard binaries and attempts to convert the selected clipboard into
     /// the binary implementation.
     ///
@@ -358,16 +373,16 @@ impl Selected {
     ///
     /// Will fail with [`ClipboardError`] when any matched display server misses it's supported
     /// clipboard binaries.
-    fn try_into_bin(&self) -> Result<Binary> {
+    fn try_into_bin(&self) -> Result<BinClipboard> {
         let bin = match self.display {
             DisplayKind::MacOs => which("pbcopy")
                 .ok()
                 .map(|t| t.as_os_str().to_owned())
-                .ok_or(ErrorKind::MissingClipboardBinary)?,
+                .ok_or(ClipboardError::MissingMacosClipboardBin)?,
             DisplayKind::Unknown => panic!("clipboard feature not supported"),
         };
         let program = ClipboardBinProgram::PbCopy;
-        Ok(Binary {
+        Ok(BinClipboard {
             bin,
             program,
             selected: self.clone(),
@@ -377,8 +392,8 @@ impl Selected {
 
 /// Not supported
 #[cfg(target_os = "windows")]
-impl Selected {
-    fn try_into_bin(&self) -> Result<Binary> {
+impl ClipboardSelected {
+    fn try_into_bin(&self) -> Result<BinClipboard> {
         Err(())
     }
 }
